import { useState, useEffect } from 'react';
export const settingsKeys = {
    viewState: 'dbg:view-state',
    loadPilets: 'dbg:load-pilets',
    hardRefresh: 'dbg:hard-refresh',
    viewOrigins: 'dbg:view-origins',
    extensionCatalogue: 'dbg:extension-catalogue',
    clearConsole: 'dbg:clear-console',
    persistSettings: 'dbg:persist-settings-data',
};
const persistKey = settingsKeys.persistSettings;
const persistSettings = !!localStorage.getItem(persistKey);
const defaultSetter = (name, value) => {
    sessionStorage.setItem(name, value);
};
const persistentSetter = (name, value) => {
    defaultSetter(name, value);
    const data = JSON.parse(localStorage.getItem(persistKey));
    data[name] = value;
    localStorage.setItem(persistKey, JSON.stringify(data));
};
function getValue(key, defaultValue, fallbackValue) {
    const value = sessionStorage.getItem(key);
    if (['on', 'off'].includes(value)) {
        return value === 'on';
    }
    else if (typeof defaultValue === 'boolean') {
        return defaultValue;
    }
    else {
        return fallbackValue;
    }
}
if (persistSettings) {
    try {
        const settings = JSON.parse(localStorage.getItem(persistKey));
        Object.keys(settings).forEach((name) => {
            const value = settings[name];
            sessionStorage.setItem(name, value);
        });
    }
    catch (_a) {
        // invalid data
        localStorage.setItem(persistKey, '{}');
    }
}
export function enablePersistance() {
    const data = {};
    const validKeys = Object.keys(settingsKeys).map((m) => settingsKeys[m]);
    for (let i = 0; i < sessionStorage.length; i++) {
        const name = sessionStorage.key(i);
        if (validKeys.includes(name)) {
            const value = sessionStorage.getItem(name);
            data[name] = value;
        }
    }
    localStorage.setItem(persistKey, JSON.stringify(data));
    return persistentSetter;
}
export function disablePersistance() {
    localStorage.removeItem(persistKey);
    return defaultSetter;
}
export const initialSetter = persistSettings ? persistentSetter : defaultSetter;
export function getInitialSettings(defaultValues) {
    return {
        viewState: getValue(settingsKeys.viewState, defaultValues.viewState, true),
        loadPilets: getValue(settingsKeys.loadPilets, defaultValues.loadPilets, false),
        hardRefresh: getValue(settingsKeys.hardRefresh, defaultValues.hardRefresh, false),
        viewOrigins: getValue(settingsKeys.viewOrigins, defaultValues.viewOrigins, false),
        extensionCatalogue: getValue(settingsKeys.extensionCatalogue, defaultValues.extensionCatalogue, true),
        clearConsole: getValue(settingsKeys.clearConsole, defaultValues.clearConsole, false),
        persistSettings,
        cataloguePath: '/$debug-extension-catalogue',
    };
}
const listeners = [];
let _navigate = undefined;
export function setNavigate(navigate) {
    _navigate = navigate;
}
export function navigate(path, state) {
    _navigate === null || _navigate === void 0 ? void 0 : _navigate(path, state);
}
let state = {
    visualize: {
        active: false,
        force: false,
    },
    catalogue: {
        active: false,
        path: '',
    },
    route: undefined,
};
export function setInitialState(initialSettings) {
    state.visualize.active = initialSettings.viewOrigins;
    state.catalogue.active = initialSettings.extensionCatalogue;
    state.catalogue.path = initialSettings.cataloguePath;
}
export function setState(dispatch) {
    const newState = dispatch(state);
    if (newState !== state) {
        state = newState;
        listeners.forEach((listener) => listener());
    }
}
export function getState() {
    return state;
}
export function subscribe(select, notify) {
    let prevState = select(state);
    const cb = () => {
        const nextState = select(state);
        if (prevState !== nextState) {
            prevState = nextState;
            notify(nextState);
        }
    };
    const unsubscribe = () => {
        const idx = listeners.indexOf(cb);
        if (idx !== -1) {
            listeners.splice(idx, 1);
        }
    };
    listeners.push(cb);
    return unsubscribe;
}
export function useDebugState(select) {
    const [state, setState] = useState(() => select(getState()));
    useEffect(() => subscribe(select, setState), []);
    return state;
}
//# sourceMappingURL=state.js.map